/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// gl_misc.c

#include "quakedef.h"

texture_t	*r_notexture_mip; // moved here from gl_main.c 
texture_t	*r_notexture_mip2; // used for non-lightmapped surfs with a missing texture

gltexture_t *particletexture;	// particle texture
gltexture_t *particletexture1;	// circle
gltexture_t *particletexture2;	// square

/*
==================
R_InitTextures
==================
*/
void	R_InitTextures (void)
{
	// create a simple checkerboard texture for the default (notexture miptex)
	r_notexture_mip = Hunk_AllocName (sizeof(texture_t), "r_notexture_mip");
	strcpy (r_notexture_mip->name, "notexture");
	r_notexture_mip->height = r_notexture_mip->width = 16;

	r_notexture_mip2 = Hunk_AllocName (sizeof(texture_t), "r_notexture_mip2");
	strcpy (r_notexture_mip2->name, "notexture2");
	r_notexture_mip2->height = r_notexture_mip2->width = 16;
}

/*
===============
ParticleTextureLookup

generate nice antialiased 32x32 circle for particles
===============
*/
int ParticleTextureLookup (int x, int y, int sharpness)
{
	int r; // distance from point x,y to circle origin, squared
	int a; // alpha value to return

	x -= 16;
	y -= 16;
	r = x * x + y * y;
	r = r > 255 ? 255 : r;
	a = sharpness * (255 - r);
	a = min(a,255);
	return a;
}

/*
===============
R_InitParticleTexture
===============
*/
void R_InitParticleTexture (void)
{
	int				x,y;
	static byte	particle1_data[64*64*4];
	static byte	particle2_data[2*2*4];
	byte			*dst;

	//
	// particle texture 1 - circle
	//
	dst = particle1_data;
	for (x=0 ; x<64 ; x++)
	{
		for (y=0 ; y<64 ; y++)
		{
			*dst++ = 255;
			*dst++ = 255;
			*dst++ = 255;
			*dst++ = ParticleTextureLookup(x, y, 8);
		}
	}

	particletexture1 = GL_LoadTexture (NULL, "particle1", 64, 64, SRC_RGBA, particle1_data, TEXPREF_PERSIST | TEXPREF_ALPHA | TEXPREF_LINEAR);

	//
	// particle texture 2 - square
	//

	dst = particle2_data;
	for (x=0 ; x<2 ; x++)
	{
		for (y=0 ; y<2 ; y++)
		{
			*dst++ = 255;
			*dst++ = 255;
			*dst++ = 255;
			*dst++ = x || y ? 0 : 255;
		}
	}

	particletexture2 = GL_LoadTexture (NULL, "particle2", 2, 2, SRC_RGBA, particle2_data, TEXPREF_PERSIST | TEXPREF_ALPHA | TEXPREF_NEAREST);
}

/*
===============
R_Init
===============
*/
void R_Init (void)
{
	Cmd_AddCommand ("timerefresh", R_TimeRefresh_f);
	Cmd_AddCommand ("pointfile", R_ReadPointFile_f);
	Cmd_AddCommand ("lightmaps", GL_LightMaps_f);

	Cmd_AddCommand ("sky", R_SkyCommand_f);
	Cmd_AddCommand ("loadsky", R_SkyCommand_f); // Nehahra
	Cmd_AddCommand ("fog", R_FogCommand_f);

	Cvar_RegisterVariable (&r_norefresh);
	Cvar_RegisterVariable (&r_fullbright);
	Cvar_RegisterVariable (&r_drawentities);
	Cvar_RegisterVariable (&r_drawworld);
	Cvar_RegisterVariable (&r_drawviewmodel);
	Cvar_RegisterVariable (&r_wateralpha);
	Cvar_RegisterVariable (&r_waterquality);
	Cvar_RegisterVariable (&r_dynamic);
	Cvar_RegisterVariable (&r_novis);
	Cvar_RegisterVariable (&r_speeds);
	Cvar_RegisterVariable (&r_flatlightstyles);
	Cvar_RegisterVariable (&r_waterwarp);
	Cvar_RegisterVariable (&r_clearcolor);
	Cvar_RegisterVariable (&r_fastsky);
	Cvar_RegisterVariable (&r_skyquality);
	Cvar_RegisterVariable (&r_skyalpha);
	Cvar_RegisterVariable (&r_skyfog);
	Cvar_RegisterVariable (&r_oldsky);

	Cvar_RegisterVariable (&gl_finish);
	Cvar_RegisterVariable (&gl_clear);
	Cvar_RegisterVariable (&gl_cull);
	Cvar_RegisterVariable (&gl_smoothmodels);
	Cvar_RegisterVariable (&gl_affinemodels);
	Cvar_RegisterVariable (&gl_polyblend);
	Cvar_RegisterVariable (&gl_flashblend);

	Cvar_RegisterVariable (&gl_fogenable);
	Cvar_RegisterVariable (&gl_fogdensity);
	Cvar_RegisterVariable (&gl_fogred);
	Cvar_RegisterVariable (&gl_foggreen);
	Cvar_RegisterVariable (&gl_fogblue);

	Cvar_RegisterVariable (&gl_bloomenable);
	Cvar_RegisterVariable (&gl_bloomdarken);
	Cvar_RegisterVariable (&gl_bloomalpha);
	Cvar_RegisterVariable (&gl_bloomintensity);
	Cvar_RegisterVariable (&gl_bloomdiamondsize);
	Cvar_RegisterVariable (&gl_bloomsamplesize);
	Cvar_RegisterVariable (&gl_bloomfastsample);

	R_InitParticles ();
	R_InitParticleTexture ();

	R_InitTranslatePlayerTextures ();

	R_InitSkyBox ();

	R_InitBloomTextures();
}

/*
===============
R_InitTranslatePlayerTextures
===============
*/
static int oldtop[MAX_SCOREBOARD]; 
static int oldbottom[MAX_SCOREBOARD];
static int oldskinnum[MAX_SCOREBOARD];
void R_InitTranslatePlayerTextures (void)
{
	int i;

	for (i = 0; i < MAX_SCOREBOARD; i++)
	{
		oldtop[i] = -1;
		oldbottom[i] = -1;
		oldskinnum[i] = -1;

		playertextures[i] = NULL; //clear playertexture pointers
	}
}

/*
===============
R_TranslatePlayerSkin

Translates a skin texture by the per-player color lookup
===============
*/
void R_TranslatePlayerSkin (int playernum)
{
	int		top, bottom;
	byte	translate[256];
	int		i, size;
	model_t	*model;
	aliashdr_t *paliashdr;
	byte	*original;
	byte	*src, *dst; 
	byte	*pixels = NULL;
	char		name[64];
	int		pixsize = 0;
//	int		mark;

//	mark = Hunk_LowMark ();

	//
	// locate the original skin pixels
	//
	currententity = &cl_entities[1+playernum];
	model = currententity->model;

	if (!model)
		return;		// player doesn't have a model yet
	if (model->type != mod_alias)
		return; // only translate skins on alias models

	paliashdr = (aliashdr_t *)Mod_Extradata (model);

	top = cl.scores[playernum].colors & 0xf0;
	bottom = (cl.scores[playernum].colors &15)<<4;

	if (!strcmp (currententity->model->name, "progs/player.mdl"))
	{
		if (top == oldtop[playernum] && bottom == oldbottom[playernum] && currententity->skinnum == oldskinnum[playernum])
			return; // translate if only player change his color
	}
	else
	{
		oldtop[playernum] = -1;
		oldbottom[playernum] = -1;
		oldskinnum[playernum] = -1;
		goto skip;
	}

	oldtop[playernum] = top;
	oldbottom[playernum] = bottom;
	oldskinnum[playernum] = currententity->skinnum;

skip:

	for (i=0 ; i<256 ; i++)
		translate[i] = i;

	for (i=0 ; i<16 ; i++)
	{
		if (top < 128)	// the artists made some backwards ranges. sigh.
			translate[TOP_RANGE+i] = top+i;
		else
			translate[TOP_RANGE+i] = top+15-i;

		if (bottom < 128)
			translate[BOTTOM_RANGE+i] = bottom+i;
		else
			translate[BOTTOM_RANGE+i] = bottom+15-i;
	}

	if (currententity->skinnum < 0 || currententity->skinnum >= paliashdr->numskins)
		original = (byte *)paliashdr + paliashdr->texels[0];
	else
		original = (byte *)paliashdr + paliashdr->texels[currententity->skinnum];

	//
	// translate texture
	//
	sprintf (name, "%s_%i_%i", currententity->model->name, currententity->skinnum, playernum);
	size = paliashdr->skinwidth * paliashdr->skinheight;

	pixels = (byte *)COM_AllocBuf ("R_TranslatePlayerSkin", pixels, &pixsize, size, 512 * 256, name);
//	pixels = Hunk_Alloc (size);

	dst = pixels;
	src = original;

	for (i=0; i<size; i++)
		*dst++ = translate[*src++];

	original = pixels;

	//upload new image
	playertextures[playernum] = GL_LoadTexture (currententity->model, name, paliashdr->skinwidth, paliashdr->skinheight, SRC_INDEXED, original, TEXPREF_PAD | TEXPREF_OVERWRITE);

	COM_FreeBuf (pixels);
//	Hunk_FreeToLowMark(mark);
}


/*
===============
R_NewMap
===============
*/
void R_NewMap (void)
{
	int		i;

	for (i=0 ; i<256 ; i++)
		d_lightstylevalue[i] = 264;		// normal light value

	memset (&r_worldentity, 0, sizeof(r_worldentity));
	r_worldentity.model = cl.worldmodel;

// clear out efrags in case the level hasn't been reloaded
// FIXME: is this one short?
	for (i=0 ; i<cl.worldmodel->numleafs ; i++)
		cl.worldmodel->leafs[i].efrags = NULL;
		 	
	r_viewleaf = NULL;
	R_ClearParticles ();

	GL_BuildLightmaps ();

	R_SkyNewMap ();
}

qboolean r_timerefresh_active;

/*
====================
R_TimeRefresh_f

For program optimization
====================
*/
void R_TimeRefresh_f (void)
{
	int			i;
	float		start, stop, time;

	glDrawBuffer  (GL_FRONT);
	glFinish ();

	r_timerefresh_active = true;
	start = Sys_DoubleTime ();
	for (i=0 ; i<128 ; i++)
	{
		r_refdef.viewangles[1] = i/128.0*360.0;
		R_RenderView ();
	}

	glFinish ();

	r_timerefresh_active = false;

	stop = Sys_DoubleTime ();
	time = stop-start;
	Con_Printf ("%f seconds (%f fps)\n", time, 128/time);

	glDrawBuffer  (GL_BACK);
	GL_EndRendering ();
}

void D_FlushCaches (void)
{
}


